//****************************************************************
//* Ce fichier et le fichier lib.h contienentles structures et   * 
//* les fonctions  que vous serez amener à utiliser. Vous n'avez * 
//* normalement pas besoin de modifier ces fichiers.             *
//**************************************************************** 

#include "lib.h"

double absd(double d){
  if(d<0) return -d;
  return d;
}

void sdl_init(){
  SDL_Init(SDL_INIT_VIDEO);
}

usint arrondi(float f){
  return (usint)(f+0.5);
}

Point::Point(){
  x=0;
  y=0;
}

Point::Point(float xx,float yy){
  x=xx;
  y=yy;
}

bool est_dans_carre(Point P,Carre c){
  float d,x,y;
  x=c.C.x;
  y=c.C.y;
  d=c.l/2;
  return P.x>=x-d and P.x<=x+d and P.y<=y+d and P.y>=y-d;
}

ostream& operator<<(ostream& os,const Point& P){
  os<<"("<<P.x<<","<<P.y<<")";
  return os;
}

Couleur::Couleur(unsigned char r,unsigned char g,unsigned char b){
  red=r;green=g;blue=b;
}

Couleur::Couleur(unsigned char g){
  red=g;green=g;blue=g;
}

Ecran_SDL::Ecran_SDL(int w,int h){
  sdl_screen=SDL_SetVideoMode(w, h, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
  rafraichir();
}

void Ecran_SDL::rafraichir(){
  SDL_Flip(sdl_screen);
}

void Ecran_SDL::dessine_point(Point P,Couleur c){
  if(P.x>=sdl_screen->w-1 or P.x<0 or P.y>=sdl_screen->h-1 or P.y<0) return;
  *((Uint32*)(sdl_screen->pixels) +  arrondi(P.x) + arrondi(P.y) * sdl_screen->w) =SDL_MapRGB(sdl_screen->format, c.red, c.green, c.blue);
}


void Ecran_SDL::dessine_segment_horizontal(Point A,usint l,Couleur c){
  for(usint x=A.x;x<=A.x+l;x++){
    dessine_point(Point(x,A.y),c);
  }
}

void Ecran_SDL::dessine_segment_vertical(Point A,usint l,Couleur c){
  for(usint y=A.y;y<=A.y+l;y++){
    dessine_point(Point(A.x,y),c);
  }
}

void Ecran_SDL::dessine_carre(Carre carre,Couleur c){
  float d,x,y;
  x=carre.C.x;
  y=carre.C.y;
  usint l=carre.l;
  d=carre.l/2;
  dessine_segment_horizontal(Point(x-d,y+d),l,c);
  dessine_segment_horizontal(Point(x-d,y-d),l,c);
  dessine_segment_vertical(Point(x+d,y-d),l,c);
  dessine_segment_vertical(Point(x-d,y-d),carre.l,c);
}

void Ecran_SDL::dessine_segment(Point A,Point B,Couleur c){
  //cout<<"A=("<<A.x<<","<<A.y<<") B=("<<B.x<<","<<B.y<<")"<<endl;
  if(A.x>sdl_screen->w or B.x>sdl_screen->w or A.y>sdl_screen->h or B.y>sdl_screen->h) return;
  int dx=B.x-A.x;
  if(dx<0){
    dessine_segment(B,A,c);
    return;
  }
  if(dx==0){
    usint ymin,ymax;
    ymin=min(A.y,B.y);
    ymax=max(A.y,B.y);
    for(usint y=ymin;y<=ymax;y++){
      dessine_point(Point(A.x,y),c);
    }
    return;
  }
  int dy=B.y-A.y;
  double m=double(dy)/double(dx);
  if(abs(m)<=1){
    double y=A.y;
    // cout<<"ici"<<endl;
    for(usint x=A.x;x<=B.x;x++){
      dessine_point(Point(x,y),c);
      y+=m;
    }
    return;
  }
  double x=A.x;
  m=1/m;
  if(dy>=0){
    for(usint y=A.y;y<=B.y;y++){
      dessine_point(Point(x,y),c);
      x+=m;
    }
    return;
  }
  for(usint y=B.y;y<=A.y;y++){
    dessine_point(Point(x,y),c);
    x+=m;
  }
}

void Ecran_SDL::effacer(){
  SDL_FillRect(sdl_screen,NULL,SDL_MapRGB(sdl_screen->format,0,0,0));
}

Evenement attendre_evenement(){
  Evenement e;
  SDL_WaitEvent(&e);
  return e;
}

bool est_quitter(Evenement& e){
  return e.type==SDL_QUIT;
}

bool est_bouton_souris_enfoncer(Evenement& e){
  return e.type==SDL_MOUSEBUTTONDOWN;
}

bool est_bouton_souris_relacher(Evenement& e){
  return e.type==SDL_MOUSEBUTTONUP;
}

bool est_deplacement_souris(Evenement& e){
  return e.type==SDL_MOUSEMOTION;
}

Point lire_position_action_bouton(Evenement& e){
  return Point(e.button.x,e.button.y);
}


Point lire_position_souris(Evenement& e){
  return Point(e.motion.x,e.motion.y);
}

void deplace_souris(Point P){
  SDL_WarpMouse(P.x,P.y);
}

