#include "matrice.h"

Matrice::Matrice(){
  nb_lignes=0;
  nb_colonnes=0;
  coeffs=NULL;
}

Matrice::Matrice(usint lignes){
  nb_lignes=lignes;
  nb_colonnes=lignes;
  coeffs=new float[nb_lignes*nb_colonnes];
  for(usint i=0;i<nb_lignes*nb_colonnes;i++){
    if(i%nb_colonnes==i/nb_colonnes){
      coeffs[i]=1;
    }
    else{
      coeffs[i]=0;
    }
  }
}

Matrice::Matrice(usint lignes, usint colonnes){
  nb_lignes=lignes;
  nb_colonnes=colonnes;
  coeffs=new float[nb_lignes*nb_colonnes];
  for(usint i=0;i<nb_lignes*nb_colonnes;i++) coeffs[i]=0;
}

Matrice::Matrice(const Matrice& M){
  nb_lignes=M.nb_lignes;
  nb_colonnes=M.nb_colonnes;
  if(M.coeffs!=NULL){
    coeffs=new float[nb_lignes*nb_colonnes];
    for(usint i=0;i<nb_lignes*nb_colonnes;i++) coeffs[i]=M.coeffs[i];
  }
}

Matrice& Matrice::operator=(const Matrice& M){
  if(coeffs!=NULL) delete[] coeffs;
  nb_lignes=M.nb_lignes;
  nb_colonnes=M.nb_colonnes;
  coeffs=new float[nb_lignes*nb_colonnes];
  for(usint i=0;i<nb_lignes*nb_colonnes;i++) coeffs[i]=M.coeffs[i];
  return *this;
}

Matrice::~Matrice(){
  if(coeffs!=NULL) delete[] coeffs;
}

usint Matrice::get_nb_lignes() const{
  return nb_lignes;
}

usint Matrice::get_nb_colonnes() const{
  return nb_colonnes;
}

Matrice::Ligne Matrice::operator[](usint l){
  if(l==0 or l>nb_lignes) throw("Numero de ligne incorrect");
  return Ligne(this,l);
}

const Matrice::Ligne Matrice::operator[](usint l) const{
  if(l==0 or l>nb_lignes) throw("Numero de ligne incorrect");
  return Ligne(this,l);
}

ostream& operator<<(ostream& os,const Matrice& M){
  os<<endl;
  for(unsigned int l=1;l<=M.nb_lignes;l++){
    os<<"["<<M[l][1];
    for(unsigned int c=2;c<=M.nb_colonnes;c++){
      os<<"\t"<<M[l][c];
    }
    os<<"]"<<endl;
  }
  return os;
}


Matrice::Ligne::Ligne(const Matrice* M,usint _l){
  matrice=M;
  l=_l;
}

float& Matrice::Ligne::operator[](usint c){
  if(c==0 or c>matrice->nb_colonnes) throw("Numero de colonne incorrect");
  return matrice->coeffs[(l-1)*matrice->nb_colonnes+(c-1)];
}

float Matrice::Ligne::operator[](usint c) const{
  if(c==0 or c>matrice->nb_colonnes) throw("Numero de colonne incorrect");
  return matrice->coeffs[(l-1)*matrice->nb_colonnes+(c-1)];
}

//*********************
//* Partie a completer
//*********************

//bool Matrice::operator==(const Matrice& B) const{}

//bool Matrice::operator!=(const Matrice& B) const{}

Matrice Matrice::operator+(const Matrice& B) const{
  //Si A et B sont des matrices
  //La commande A+B appel la fonction A.operator+(B) ou B est un
  //paramètre explicite et A désigne la matrice courante
  const Matrice& A=*this;

  if(A.get_nb_lignes()!=B.get_nb_lignes() or A.get_nb_colonnes()!=B.get_nb_colonnes()){
    throw("Tailles de matrices différentes");
  }
  //Création d'un matrice à A.get_nb_lignes() lignes et A.get_nb_colonnes() colonnes 
  Matrice R(A.get_nb_lignes(),A.get_nb_colonnes()); 
  for(usint l=1;l<=A.get_nb_lignes();l++){
    for(usint c=1;c<=A.get_nb_colonnes();c++){
      R[l][c]=A[l][c]+B[l][c];
    }
  }
  return R;
}

Matrice Matrice::operator-(const Matrice& B) const{
  const Matrice& A=*this;
  if(A.get_nb_lignes()!=B.get_nb_lignes() or A.get_nb_colonnes()!=B.get_nb_colonnes()){
    throw("Tailles de matrices différentes");
  }
  Matrice R(A.get_nb_lignes(),A.get_nb_colonnes()); 
  for(usint l=1;l<=A.get_nb_lignes();l++){
    for(usint c=1;c<=A.get_nb_colonnes();c++){
      R[l][c]=A[l][c]-B[l][c];
    }
  }
  return R;
}

Matrice Matrice::operator*(const float& x) const{
  const Matrice& A=*this;
  Matrice R(A.get_nb_lignes(),A.get_nb_colonnes());
  for(usint l=1;l<=A.get_nb_lignes();l++){
    for(usint c=1;c<=A.get_nb_colonnes();c++){
      R[l][c]=x*A[l][c];
    }
  }
  return R;
}

Matrice operator*(const float& x,const Matrice& M){
  return M*x;
}

Matrice Matrice::operator*(const Matrice& B) const{
  const Matrice& A=*this;
  if(A.get_nb_colonnes()!=B.get_nb_lignes()){
    throw("Taille incorrecte");
  }
  usint i,j,k;
  float s;
  Matrice R(A.get_nb_lignes(),B.get_nb_colonnes());
  for(i=1;i<=A.get_nb_lignes();i++){
    for(j=1;j<=B.get_nb_colonnes();j++){
      s=0;
      for(k=1;k<=A.get_nb_colonnes();k++){
	s=s+A[i][k]*B[k][j];
      }
      R[i][j]=s;
    }
  }
  return R;
}

void Matrice::ech_ligne(usint i,usint j){ 
  Matrice& A=*this;
  float temp;
  for(usint c=1;c<=A.get_nb_colonnes();c++){
    temp=A[i][c];
    A[i][c]=A[j][c];
    A[j][c]=temp;
  }

}

void Matrice::mul_ligne(usint i,float r){
  Matrice& A=*this;
  for(usint c=1;c<=A.get_nb_colonnes();c++){
    A[i][c]=r*A[i][c];
  }
}

void Matrice::add_mul_ligne(usint i,usint j, float r){
  Matrice& A=*this;
  for(usint c=1;c<=A.get_nb_colonnes();c++){
    A[i][c]=A[i][c]+r*A[j][c];
  }
}

Matrice Matrice::Gauss() const{
  const Matrice& A=*this;
  Matrice R(A);
  usint nc,nl;
  nc=A.get_nb_colonnes();
  nl=A.get_nb_lignes();
  usint lp=1; 
  for(usint cp=1;cp<=nc;cp++){
    for(usint l=lp;l<=nl;l++){
      if(R[l][cp]!=0){
	R.ech_ligne(lp,l);
	R.mul_ligne(lp,1/R[lp][cp]);
	for(usint ll=1;ll<=nl;ll++){
	  if(ll!=lp){
	    R.add_mul_ligne(ll,lp,-R[ll][cp]);
	  }
	}
	lp++;
	break;
      }
    }
  }
  return R;
}

usint Matrice::rang() const{
  const Matrice& A=*this;
  Matrice B=A.Gauss();
  usint l,c,nl,nc;
  nl=A.get_nb_lignes();
  nc=A.get_nb_colonnes();
  //B est echelonnée réduite
  for(l=1;l<=nl;l++){
    for(c=1;c<=nc;c++){
      if(B[l][c]!=0) break;
    }
    if(c==nc+1){//la ligne l est nulle
      return l-1;
    }
  }
  return l-1;
}

float Matrice::determinant() const{
  bool pivot_trouve;
  const Matrice& A=*this;
  Matrice R(A);
  float det=1;
  usint nc,nl;
  nc=A.get_nb_colonnes();
  nl=A.get_nb_lignes();
  if(nc!=nl) throw("Determinant non defini");
  usint lp=1; 
  for(usint cp=1;cp<=nc;cp++){
    pivot_trouve=false;
    for(usint l=lp;l<=nl;l++){
      if(R[l][cp]!=0){
	pivot_trouve=true;
	R.ech_ligne(lp,l);
	if(l!=lp) det=-det;
	det=det*R[lp][cp];
	R.mul_ligne(lp,1/R[lp][cp]);
	for(usint ll=1;ll<=nl;ll++){
	  if(ll!=lp){
	    R.add_mul_ligne(ll,lp,-R[ll][cp]);
	  }
	}
	lp++;
	break;
      }
    }
    if(not pivot_trouve) return 0;
  }
  return det;
}


bool Matrice::est_inversible() const{
  const Matrice& A=*this;
  if(A.get_nb_lignes()!=A.get_nb_colonnes()){
    return false;
  }
  return A.determinant()!=0;
}

Matrice Matrice::inverse() const{
  bool pivot_trouve;
  const Matrice& A=*this;
  Matrice R(A);
  float det=1;
  float temp;
  usint nc,nl;
  nc=A.get_nb_colonnes();
  nl=A.get_nb_lignes();
  if(nc!=nl) throw("Matrice non inversible");
  Matrice I(nc);
  usint lp=1; 
  for(usint cp=1;cp<=nc;cp++){
    pivot_trouve=false;
    for(usint l=lp;l<=nl;l++){
      if(R[l][cp]!=0){
	pivot_trouve=true;
	R.ech_ligne(lp,l);
	I.ech_ligne(lp,l);
	if(l!=lp) det=-det;
	det=det*R[lp][cp];
	temp=1/R[lp][cp];
	R.mul_ligne(lp,temp);
	I.mul_ligne(lp,temp);
	for(usint ll=1;ll<=nl;ll++){
	  if(ll!=lp){
	    temp=-R[ll][cp];
	    R.add_mul_ligne(ll,lp,temp);
	    I.add_mul_ligne(ll,lp,temp);
	  }
	}
	lp++;
	break;
      }
    }
    if(not pivot_trouve) throw("Matrice non inversible");
  }
  return I;
}

